@@ -72,15 +72,16 @@ private Q_SLOTS:
 
     // Building
     void slotSelectTarget();
-    void slotBuildActiveTarget();
+    void slotBuildSelectedTarget();
+    void slotBuildAndRunSelectedTarget();
     void slotBuildPreviousTarget();
-    void slotBuildDefaultTarget();
     bool slotStop();
 
     // Parse output
     void slotProcExited(int exitCode, QProcess::ExitStatus exitStatus);
     void slotReadReadyStdErr();
     void slotReadReadyStdOut();
+    void slotRunAfterBuild();
 
     // Selecting warnings/errors
     void slotNext();
@@ -108,7 +109,6 @@ private Q_SLOTS:
     void slotPluginViewCreated(const QString &name, QObject *pluginView);
     void slotPluginViewDeleted(const QString &name, QObject *pluginView);
     void slotProjectMapChanged();
-    void slotAddProjectTarget();
 
 protected:
     bool eventFilter(QObject *obj, QEvent *ev) override;
@@ -129,6 +129,8 @@ private:
     void clearMarks();
     void addMarks(KTextEditor::Document *doc, bool mark);
 
+    void addProjectTarget();
+
     KTextEditor::MainWindow *m_win;
     QWidget *m_toolView;
     Ui::build m_buildUi{};
@@ -140,6 +142,7 @@ private:
     QString m_stdErr;
     QString m_currentlyBuildingTarget;
     bool m_buildCancelled;
+    bool m_runAfterBuild = false;
     int m_displayModeBeforeBuild;
     QString m_make_dir;
     QStack<QString> m_make_dir_stack;
@@ -149,11 +152,12 @@ private:
     QRegularExpression m_newDirDetector;
     unsigned int m_numErrors = 0;
     unsigned int m_numWarnings = 0;
-    QString m_prevItemContent;
-    QModelIndex m_previousIndex;
+    QPersistentModelIndex m_previousIndex;
     QPointer<KTextEditor::Message> m_infoMessage;
     QPointer<QAction> m_showMarks;
     QHash<KTextEditor::Document *, QPointer<KTextEditor::Document>> m_markedDocs;
+    int m_projectTargetsetRow = 0;
+    bool m_firstBuild = true;
 
     /**
      * current project plugin view, if any
