@@ -7,6 +7,7 @@
 
 #include "kateprojectpluginview.h"
 #include "fileutil.h"
+#include "gitprocess.h"
 #include "gitwidget.h"
 #include "kateproject.h"
 #include "kateprojectinfoview.h"
@@ -21,20 +22,19 @@
 #include <ktexteditor/editor.h>
 #include <ktexteditor/view.h>
 
-#include <KAboutData>
 #include <KActionCollection>
 #include <KActionMenu>
 #include <KLocalizedString>
 #include <KPluginFactory>
+#include <KStandardAction>
 #include <KStringHandler>
 #include <KXMLGUIFactory>
+#include <KXmlGuiWindow>
 
 #include <QAction>
-#include <QDialog>
 #include <QFileDialog>
 #include <QHBoxLayout>
 #include <QKeyEvent>
-#include <QVBoxLayout>
 
 #define PROJECTCLOSEICON "window-close"
 
@@ -51,7 +51,7 @@ KateProjectPluginView::KateProjectPluginView(KateProjectPlugin *plugin, KTextEdi
     , m_gotoSymbolAction(nullptr)
     , m_gotoSymbolActionAppMenu(nullptr)
 {
-    KXMLGUIClient::setComponentName(QStringLiteral("kateproject"), i18n("Kate Project Manager"));
+    KXMLGUIClient::setComponentName(QStringLiteral("kateproject"), i18n("Project Manager"));
     setXMLFile(QStringLiteral("ui.rc"));
 
     /**
@@ -62,11 +62,7 @@ KateProjectPluginView::KateProjectPluginView(KateProjectPlugin *plugin, KTextEdi
                                               KTextEditor::MainWindow::Left,
                                               QIcon::fromTheme(QStringLiteral("project-open")),
                                               i18n("Projects"));
-    m_gitToolView.reset(m_mainWindow->createToolView(m_plugin,
-                                                     QStringLiteral("kateprojectgit"),
-                                                     KTextEditor::MainWindow::Left,
-                                                     QIcon(QStringLiteral(":/icons/icons/sc-apps-git.svg")),
-                                                     i18n("Git")));
+    m_gitToolView.reset(m_mainWindow->createToolView(m_plugin, QStringLiteral("kateprojectgit"), KTextEditor::MainWindow::Left, gitIcon(), i18n("Git")));
     m_toolInfoView = m_mainWindow->createToolView(m_plugin,
                                                   QStringLiteral("kateprojectinfo"),
                                                   KTextEditor::MainWindow::Bottom,
@@ -81,6 +77,7 @@ KateProjectPluginView::KateProjectPluginView(KateProjectPlugin *plugin, KTextEdi
     m_reloadButton = new QToolButton(m_toolView);
     m_reloadButton->setAutoRaise(true);
     m_reloadButton->setIcon(QIcon::fromTheme(QStringLiteral("view-refresh")));
+    m_reloadButton->setToolTip(i18n("Reload project"));
     m_closeProjectButton = new QToolButton(m_toolView);
     m_closeProjectButton->setAutoRaise(true);
     m_closeProjectButton->setIcon(QIcon::fromTheme(QStringLiteral(PROJECTCLOSEICON)));
@@ -97,6 +94,7 @@ KateProjectPluginView::KateProjectPluginView(KateProjectPlugin *plugin, KTextEdi
     m_gitStatusRefreshButton = new QToolButton(m_gitToolView.get());
     m_gitStatusRefreshButton->setAutoRaise(true);
     m_gitStatusRefreshButton->setIcon(QIcon::fromTheme(QStringLiteral("view-refresh")));
+    m_gitStatusRefreshButton->setToolTip(i18n("Refresh git status"));
     layout = new QHBoxLayout();
     layout->setSpacing(0);
     layout->addWidget(m_projectsComboGit);
@@ -173,6 +171,8 @@ KateProjectPluginView::KateProjectPluginView(KateProjectPlugin *plugin, KTextEdi
      */
     auto a = actionCollection()->addAction(QStringLiteral("projects_open_project"), this, SLOT(openDirectoryOrProject()));
     a->setText(i18n("Open Folder..."));
+    a->setIcon(QIcon::fromTheme(QStringLiteral("document-open-folder")));
+    actionCollection()->setDefaultShortcut(a, QKeySequence(QKeySequence(QStringLiteral("Ctrl+T, O"), QKeySequence::PortableText)));
 
     m_projectTodosAction = a = actionCollection()->addAction(QStringLiteral("projects_todos"), this, SLOT(showProjectTodos()));
     a->setText(i18n("Project TODOs"));
@@ -292,8 +292,7 @@ void KateProjectPluginView::slotConfigUpdated()
     }
 
     // update action state
-    m_gotoSymbolActionAppMenu->setEnabled(m_toolMultiView);
-    m_gotoSymbolAction->setEnabled(m_toolMultiView);
+    updateActions();
 }
 
 QPair<KateProjectView *, KateProjectInfoView *> KateProjectPluginView::viewForProject(KateProject *project)
@@ -326,6 +325,12 @@ QPair<KateProjectView *, KateProjectInfoView *> KateProjectPluginView::viewForPr
     m_stackedGitViews->addWidget(gitView);
     m_projectsCombo->addItem(QIcon::fromTheme(QStringLiteral("project-open")), project->name(), project->fileName());
     m_projectsComboGit->addItem(QIcon::fromTheme(QStringLiteral("project-open")), project->name(), project->fileName());
+    connect(project, &KateProject::projectMapChanged, this, [this] {
+        auto widget = m_stackedProjectViews->currentWidget();
+        if (widget && static_cast<KateProjectView *>(widget)->project() == sender()) {
+            Q_EMIT projectMapChanged();
+        }
+    });
 
     /*
      * inform onward
@@ -578,6 +583,16 @@ void KateProjectPluginView::switchToProject(const QDir &dir)
     }
 }
 
+void KateProjectPluginView::runCmdInTerminal(const QString &workingDir, const QString &cmd)
+{
+    m_mainWindow->showToolView(m_toolInfoView);
+    auto widget = static_cast<KateProjectInfoView *>(m_stackedProjectInfoViews->currentWidget());
+    if (!widget) {
+        return;
+    }
+    widget->runCmdInTerminal(workingDir, cmd);
+}
+
 void KateProjectPluginView::slotViewCreated(KTextEditor::View *view)
 {
     /**
@@ -790,7 +805,10 @@ void KateProjectPluginView::slotUpdateStatus(bool visible)
     if (auto widget = static_cast<GitWidget *>(m_stackedGitViews->currentWidget())) {
         // To support separate-git-dir always use dotGitPath
         // We need to add the path every time again because it's always a different file
-        m_gitChangedWatcher.addPath(widget->dotGitPath() + QStringLiteral(".git/index"));
+        const auto path = widget->indexPath();
+        if (!path.isEmpty()) {
+            m_gitChangedWatcher.addPath(path);
+        }
         widget->updateStatus();
     }
 }
@@ -803,9 +821,27 @@ void KateProjectPluginView::openDirectoryOrProject()
         return;
     }
 
+    openDirectoryOrProject(dir);
+}
+
+void KateProjectPluginView::openDirectoryOrProject(const QDir &dir)
+{
     // switch to this project if there
     if (auto project = m_plugin->projectForDir(dir, true)) {
+        // just activate the right plugin in the toolview
         slotActivateProject(project);
+
+        // this is a user action, ensure the toolview is visible
+        mainWindow()->showToolView(m_toolView);
+
+        // add the project to the recently opened items list
+        if (auto *parentClient = qobject_cast<KXmlGuiWindow *>(m_mainWindow->window())) {
+            if (auto *openRecentAction = parentClient->action(KStandardAction::name(KStandardAction::StandardAction::OpenRecent))) {
+                if (auto *recentFilesAction = qobject_cast<KRecentFilesAction *>(openRecentAction)) {
+                    recentFilesAction->addUrl(QUrl::fromLocalFile(dir.path()));
+                }
+            }
+        }
     }
 }
 
@@ -819,23 +855,6 @@ void KateProjectPluginView::showProjectTodos()
     pgrep->exec(nullptr, QStringLiteral("preg (TODO|FIXME)\\b"), msg);
 }
 
-void KateProjectPluginView::showDiffInFixedView(const QByteArray &contents)
-{
-    if (!m_fixedView.view) {
-        m_fixedView.view = mainWindow()->openUrl(QUrl());
-        m_fixedView.defaultMenu = m_fixedView.view->contextMenu();
-    }
-
-    m_fixedView.view->document()->setText(QString::fromUtf8(contents));
-    m_fixedView.view->document()->setHighlightingMode(QStringLiteral("Diff"));
-    /** We don't want save dialog on close */
-    m_fixedView.view->document()->setModified(false);
-    m_fixedView.view->setCursorPosition({0, 0});
-    m_fixedView.restoreMenu();
-    /** Activate this view */
-    m_mainWindow->activateView(m_fixedView.view->document());
-}
-
 void KateProjectPluginView::openTerminal(const QString &dirPath, KateProject *project)
 {
     m_mainWindow->showToolView(m_toolInfoView);
@@ -845,22 +864,9 @@ void KateProjectPluginView::openTerminal(const QString &dirPath, KateProject *pr
     }
 }
 
-QString KateProjectPluginView::projectBaseDirForUrl(const QUrl &url)
-{
-    const auto projects = m_plugin->projects();
-    QString ret;
-    for (auto *project : projects) {
-        const QString dir = project->baseDir();
-        if (url.toLocalFile().startsWith(dir)) {
-            // prefer the longer base path
-            ret = ret.length() < dir.length() ? dir : ret;
-        }
-    }
-    return ret;
-}
-
 void KateProjectPluginView::updateActions()
 {
+    const bool hasMultipleProjects = m_projectsCombo->count() > 1;
     // currently some project active?
     const bool projectActive = !projectBaseDir().isEmpty();
     m_projectsCombo->setEnabled(projectActive);
@@ -868,16 +874,19 @@ void KateProjectPluginView::updateActions()
     m_reloadButton->setEnabled(projectActive);
     m_closeProjectButton->setEnabled(projectActive);
     m_gitStatusRefreshButton->setEnabled(projectActive);
-    m_lookupAction->setEnabled(projectActive);
-    m_gotoSymbolAction->setEnabled(projectActive);
-    m_gotoSymbolActionAppMenu->setEnabled(projectActive);
     m_projectTodosAction->setEnabled(projectActive);
-    m_projectPrevAction->setEnabled(projectActive);
-    m_projectNextAction->setEnabled(projectActive);
-    m_projectGotoIndexAction->setEnabled(projectActive);
+    m_projectPrevAction->setEnabled(projectActive && hasMultipleProjects);
+    m_projectNextAction->setEnabled(projectActive && hasMultipleProjects);
     m_projectCloseAction->setEnabled(projectActive);
-    m_projectCloseAllAction->setEnabled(m_projectsCombo->count() > 0);
+    m_projectCloseAllAction->setEnabled(hasMultipleProjects);
     m_projectCloseWithoutDocumentsAction->setEnabled(m_projectsCombo->count() > 0);
+
+    const bool hasIndex = projectActive && m_plugin->getIndexEnabled();
+    m_lookupAction->setVisible(hasIndex);
+    m_gotoSymbolAction->setVisible(hasIndex);
+    m_projectGotoIndexAction->setVisible(hasIndex);
+    m_gotoSymbolActionAppMenu->setVisible(hasIndex);
+    actionCollection()->action(QStringLiteral("popup_project"))->setVisible(hasIndex);
 }
 
 void KateProjectPluginView::slotActivateProject(KateProject *project)
@@ -885,7 +894,6 @@ void KateProjectPluginView::slotActivateProject(KateProject *project)
     const int index = m_projectsCombo->findData(project->fileName());
     if (index >= 0) {
         m_projectsCombo->setCurrentIndex(index);
-        mainWindow()->showToolView(m_toolView);
     }
 }
 
