@@ -10,6 +10,7 @@
 #include "kateprojectfiltermodel.h"
 #include "kateprojectpluginview.h"
 #include "kateprojecttreeviewcontextmenu.h"
+#include "ktexteditor_utils.h"
 
 #include <KTextEditor/Document>
 #include <KTextEditor/MainWindow>
@@ -32,6 +33,9 @@ KateProjectViewTree::KateProjectViewTree(KateProjectPluginView *pluginView, Kate
     setAllColumnsShowFocus(true);
     setIndentation(12);
 
+    setDragDropMode(QAbstractItemView::DropOnly);
+    setDragDropOverwriteMode(false);
+
     /**
      * attach view => project
      * do this once, model is stable for whole project life time
@@ -132,7 +136,7 @@ void KateProjectViewTree::addFile(const QModelIndex &idx, const QString &fileNam
         genericMessage.insert(QStringLiteral("category"), i18n("Project"));
         genericMessage.insert(QStringLiteral("categoryIcon"), QIcon::fromTheme(QStringLiteral("document-new")));
         genericMessage.insert(QStringLiteral("text"), i18n("Failed to create file: %1, Error: %2", fileName, f.errorString()));
-        Q_EMIT m_pluginView->message(genericMessage);
+        Utils::showMessage(genericMessage, m_pluginView->mainWindow());
         return;
     }
 
@@ -157,7 +161,7 @@ void KateProjectViewTree::addDirectory(const QModelIndex &idx, const QString &na
         genericMessage.insert(QStringLiteral("category"), i18n("Project"));
         genericMessage.insert(QStringLiteral("categoryIcon"), QIcon::fromTheme(QStringLiteral("folder-new")));
         genericMessage.insert(QStringLiteral("text"), i18n("Failed to create dir: %1", name));
-        Q_EMIT m_pluginView->message(genericMessage);
+        Utils::showMessage(genericMessage, m_pluginView->mainWindow());
         return;
     }
 
@@ -247,7 +251,6 @@ void KateProjectViewTree::contextMenuEvent(QContextMenuEvent *event)
     }
 
     KateProjectTreeViewContextMenu menu;
-    connect(&menu, &KateProjectTreeViewContextMenu::showFileHistory, this, &KateProjectViewTree::showFileHistory);
     menu.exec(filePath, index, viewport()->mapToGlobal(event->pos()), this);
 
     event->accept();
