@@ -29,7 +29,7 @@
 #include <KMessageBox>
 #include <KPluralHandlingSpinBox>
 #include <KSharedConfig>
-#include <KStandardAction>
+#include <kwidgetsaddons_version.h>
 
 #include <kwidgetsaddons_version.h>
 
@@ -40,22 +40,59 @@
 #include <QFrame>
 #include <QGroupBox>
 #include <QLabel>
+#include <QLineEdit>
+#include <QListView>
+#include <QPainter>
 #include <QScreen>
 #include <QScrollArea>
 #include <QScrollBar>
+#include <QTimer>
 #include <QVBoxLayout>
 
 KateConfigDialog::KateConfigDialog(KateMainWindow *parent)
     : KPageDialog(parent)
     , m_mainWindow(parent)
+    , m_searchLineEdit(new QLineEdit(this))
+    , m_searchTimer(new QTimer(this))
 {
     setWindowTitle(i18n("Configure"));
+    setWindowIcon(QIcon::fromTheme(QStringLiteral("configure")));
     setStandardButtons(QDialogButtonBox::Ok | QDialogButtonBox::Apply | QDialogButtonBox::Cancel | QDialogButtonBox::Help);
 
-    // we may have a lot of pages, we want small icons for the list
+    m_searchLineEdit->setPlaceholderText(i18n("Search..."));
+    setFocusProxy(m_searchLineEdit);
+    m_searchTimer->setSingleShot(true);
+    m_searchTimer->setInterval(400);
+    m_searchTimer->callOnTimeout(this, &KateConfigDialog::onSearchTextChanged);
+    connect(m_searchLineEdit, &QLineEdit::textChanged, m_searchTimer, qOverload<>(&QTimer::start));
+
+    if (auto layout = qobject_cast<QVBoxLayout *>(this->layout())) {
+        layout->insertWidget(0, m_searchLineEdit);
+    } else {
+        m_searchLineEdit->hide();
+        qWarning() << Q_FUNC_INFO << "Failed to get layout! Search will be disabled";
+    }
+
+    // we may have a lot of pages on Kate, we want small icons for the list
+    if (KateApp::isKate()) {
 #if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 94, 0)
-    setFaceType(KPageDialog::FlatList);
+        setFaceType(KPageDialog::FlatList);
 #endif
+    }
+    else {
+        setFaceType(KPageDialog::List);
+    }
+
+    const auto listViews = findChildren<QListView *>();
+    for (auto *lv : listViews) {
+        if (qstrcmp(lv->metaObject()->className(), "KDEPrivate::KPageListView") == 0) {
+            m_sideBar = lv;
+            break;
+        }
+    }
+    if (!m_sideBar) {
+        qWarning() << "Unable to find config dialog sidebar listview!!";
+    }
 
     // first: add the KTextEditor config pages
     // rational: most people want to alter e.g. the fonts, the colors or some other editor stuff first
@@ -81,18 +118,221 @@ KateConfigDialog::KateConfigDialog(KateMainWindow *parent)
     connect(this, &KateConfigDialog::accepted, this, &KateConfigDialog::slotApply);
     connect(buttonBox()->button(QDialogButtonBox::Apply), &QPushButton::clicked, this, &KateConfigDialog::slotApply);
     connect(buttonBox()->button(QDialogButtonBox::Help), &QPushButton::clicked, this, &KateConfigDialog::slotHelp);
+
+    button(QDialogButtonBox::Ok)->hide();
+    button(QDialogButtonBox::Cancel)->setText(i18n("Close"));
+    button(QDialogButtonBox::Apply)->setText(i18n("Save"));
+
+    // set focus on next iteration of event loop, doesn't work directly for some reason
+    QMetaObject::invokeMethod(
+        this,
+        [this] {
+            m_searchLineEdit->setFocus();
+        },
+        Qt::QueuedConnection);
 }
 
-QSize KateConfigDialog::sizeHint() const
+template<typename WidgetType>
+QVector<QWidget *> hasMatchingText(const QString &text, QWidget *page)
 {
-    // start with a bit enlarged default size hint to minimize changes of useless scrollbars
-    QSize size = KPageDialog::sizeHint() * 1.3;
+    QVector<QWidget *> ret;
+    const auto widgets = page->findChildren<WidgetType *>();
+    for (auto label : widgets) {
+        if (label->text().contains(text, Qt::CaseInsensitive)) {
+            ret << label;
+        }
+    }
+    return ret;
+}
 
-    // enlarge it to half of the main window size, if that is larger
-    size = size.expandedTo(m_mainWindow->size() * 0.5);
+template<>
+QVector<QWidget *> hasMatchingText<QComboBox>(const QString &text, QWidget *page)
+{
+    QVector<QWidget *> ret;
+    const auto comboxBoxes = page->findChildren<QComboBox *>();
+    for (auto cb : comboxBoxes) {
+        if (cb->findText(text, Qt::MatchFlag::MatchContains) != -1) {
+            ret << cb;
+        }
+    }
+    return ret;
+}
 
-    // return bounded size to available real screen space
-    return size.boundedTo(screen()->availableSize() * 0.9);
+template<typename...>
+struct FindChildrenHelper {
+    static QVector<QWidget *> hasMatchingTextForTypes(const QString &, QWidget *)
+    {
+        return {};
+    }
+};
+
+template<typename First, typename... Rest>
+struct FindChildrenHelper<First, Rest...> {
+    static QVector<QWidget *> hasMatchingTextForTypes(const QString &text, QWidget *page)
+    {
+        return hasMatchingText<First>(text, page) << FindChildrenHelper<Rest...>::hasMatchingTextForTypes(text, page);
+    }
+};
+
+class SearchMatchOverlay : public QWidget
+{
+public:
+    SearchMatchOverlay(QWidget *parent, int tabIdx = -1)
+        : QWidget(parent)
+        , m_tabIdx(tabIdx)
+    {
+        setAttribute(Qt::WA_TransparentForMouseEvents, true);
+        resize_impl();
+        parent->installEventFilter(this);
+
+        show();
+        raise();
+    }
+
+    void doResize()
+    {
+        // trigger it delayed just to be safe as we will be calling
+        // it in response to Resize sometimes
+        QMetaObject::invokeMethod(this, &SearchMatchOverlay::resize_impl, Qt::QueuedConnection);
+    }
+
+    void resize_impl()
+    {
+        if (m_tabIdx >= 0) {
+            auto tabBar = qobject_cast<QTabBar *>(parentWidget());
+            if (!tabBar) {
+                setVisible(false);
+                return;
+            }
+            const QRect r = tabBar->tabRect(m_tabIdx);
+            if (geometry() != r) {
+                setGeometry(r);
+            }
+            return;
+        }
+
+        if (parentWidget() && size() != parentWidget()->size()) {
+            resize(parentWidget()->size());
+        }
+    }
+
+    bool eventFilter(QObject *o, QEvent *e) override
+    {
+        if (parentWidget() && o == parentWidget() && (e->type() == QEvent::Resize || e->type() == QEvent::Show)) {
+            doResize();
+        }
+        return QWidget::eventFilter(o, e);
+    }
+
+protected:
+    void paintEvent(QPaintEvent *e) override
+    {
+        QPainter p(this);
+        p.setClipRegion(e->region());
+        QColor c = palette().brush(QPalette::Active, QPalette::Highlight).color();
+        c.setAlpha(110);
+        p.fillRect(rect(), c);
+    }
+
+private:
+    int m_tabIdx = -1;
+};
+
+void KateConfigDialog::onSearchTextChanged()
+{
+    if (!m_sideBar) {
+        qWarning() << Q_FUNC_INFO << "No config dialog sidebar, search will not continue";
+        return;
+    }
+
+    const QString text = m_searchLineEdit->text();
+    QSet<QString> pagesToHide;
+    QVector<QWidget *> matchedWidgets;
+    if (!text.isEmpty()) {
+        for (auto item : std::as_const(m_allPages)) {
+            const auto matchingWidgets = FindChildrenHelper<QLabel, QAbstractButton, QComboBox>::hasMatchingTextForTypes(text, item->widget());
+            if (matchingWidgets.isEmpty()) {
+                pagesToHide << item->name();
+            }
+            matchedWidgets << matchingWidgets;
+        }
+    }
+
+    if (auto model = m_sideBar->model()) {
+        QModelIndex current;
+        for (int i = 0; i < model->rowCount(); ++i) {
+            const auto itemName = model->index(i, 0).data().toString();
+            m_sideBar->setRowHidden(i, pagesToHide.contains(itemName));
+            if (!text.isEmpty() && !m_sideBar->isRowHidden(i) && !current.isValid()) {
+                current = model->index(i, 0);
+            }
+        }
+        if (current.isValid()) {
+            m_sideBar->setCurrentIndex(current);
+        }
+    }
+
+    qDeleteAll(m_searchMatchOverlays);
+    m_searchMatchOverlays.clear();
+
+    using TabWidgetAndPage = QPair<QTabWidget *, QWidget *>;
+    auto tabWidgetParent = [](QWidget *w) {
+        // Finds if @p w is in a QTabWidget and returns
+        // The QTabWidget + the widget in the stack where
+        // @p w lives
+        auto parent = w->parentWidget();
+        TabWidgetAndPage p = {nullptr, nullptr};
+        if (auto tw = qobject_cast<QTabWidget *>(parent)) {
+            p.first = tw;
+        }
+        QVarLengthArray<QWidget *, 8> parentChain;
+        while (parent) {
+            if (!p.first) {
+                if (auto tw = qobject_cast<QTabWidget *>(parent)) {
+                    if (parentChain.size() >= 3) {
+                        // last == QTabWidget
+                        // second last == QStackedWidget of QTabWidget
+                        // third last => the widget we want
+                        p.second = parentChain.value((parentChain.size() - 1) - 2);
+                    }
+                    p.first = tw;
+                    break;
+                }
+            }
+            parent = parent->parentWidget();
+            parentChain << parent;
+        }
+        return p;
+    };
+
+    for (auto w : std::as_const(matchedWidgets)) {
+        if (w) {
+            m_searchMatchOverlays << new SearchMatchOverlay(w);
+
+            if (!w->isVisible()) {
+                const auto [tabWidget, page] = tabWidgetParent(w);
+                if (!tabWidget && !page) {
+                    continue;
+                }
+                const int idx = tabWidget->indexOf(page);
+                if (idx < 0) {
+                    //                     qDebug() << page << tabWidget << "not found" << w;
+                    continue;
+                }
+                m_searchMatchOverlays << new SearchMatchOverlay(tabWidget->tabBar(), idx);
+            }
+        }
+    }
+}
+
+KateConfigDialog *KateConfigDialog::widget(KateMainWindow *mw)
+{
+    static QPointer<KateConfigDialog> dlg = nullptr;
+    if (!dlg) {
+        dlg = new KateConfigDialog(mw);
+        return dlg;
+    }
+    return dlg;
 }
 
 void KateConfigDialog::addBehaviorPage()
@@ -102,6 +342,7 @@ void KateConfigDialog::addBehaviorPage()
 
     QFrame *generalFrame = new QFrame;
     KPageWidgetItem *item = addScrollablePage(generalFrame, i18n("Behavior"));
+    m_allPages.insert(item);
     item->setHeader(i18n("Behavior Options"));
     item->setIcon(QIcon::fromTheme(QStringLiteral("preferences-system-windows-behavior")));
 
@@ -144,6 +385,7 @@ void KateConfigDialog::addBehaviorPage()
     // modified files notification
     m_modNotifications = new QCheckBox(i18n("Use a separate &dialog for handling externally modified files"), buttonGroup);
     m_modNotifications->setChecked(m_mainWindow->modNotificationEnabled());
+    m_modNotifications->setCursor(Qt::WhatsThisCursor);
     m_modNotifications->setWhatsThis(
         i18n("If enabled, a modal dialog will be used to show all of the modified files. "
              "If not enabled, you will be individually asked what to do for each modified file "
@@ -151,10 +393,21 @@ void KateConfigDialog::addBehaviorPage()
     connect(m_modNotifications, &QCheckBox::toggled, this, &KateConfigDialog::slotChanged);
 
     vbox->addWidget(m_modNotifications);
+    buttonGroup->setLayout(vbox);
 
-    // show text for the left and right sidebars?
-    // only useful for Kate
     if (KateApp::isKate()) {
+        QGroupBox *buttonGroup = new QGroupBox(i18n("&Sidebars"), generalFrame);
+        layout->addWidget(buttonGroup);
+        QVBoxLayout *vbox = new QVBoxLayout;
+        m_syncSectionSizeWithSidebarTabs = new QCheckBox(i18n("Sync section size with tab positions"), buttonGroup);
+        m_syncSectionSizeWithSidebarTabs->setChecked(cgGeneral.readEntry("Sync section size with tab positions", false));
+        m_syncSectionSizeWithSidebarTabs->setCursor(Qt::WhatsThisCursor);
+        m_syncSectionSizeWithSidebarTabs->setWhatsThis(
+            i18n("When enabled the section size will be determined by the position of the tabs.\n"
+                 "This option does not affect the bottom sidebar."));
+        connect(m_syncSectionSizeWithSidebarTabs, &QCheckBox::toggled, this, &KateConfigDialog::slotChanged);
+        vbox->addWidget(m_syncSectionSizeWithSidebarTabs);
+
         m_showTextForLeftRightSidebars = new QCheckBox(i18n("Show text for left and right sidebar buttons"), buttonGroup);
         m_showTextForLeftRightSidebars->setChecked(cgGeneral.readEntry("Show text for left and right sidebar", false));
         connect(m_showTextForLeftRightSidebars, &QCheckBox::toggled, this, &KateConfigDialog::slotChanged);
@@ -175,10 +428,9 @@ void KateConfigDialog::addBehaviorPage()
             m_leftRightSidebarsIconSize->setEnabled(!v);
             l->setEnabled(!v);
         });
+        buttonGroup->setLayout(vbox);
     }
 
-    buttonGroup->setLayout(vbox);
-
     // tabbar => we allow to configure some limit on number of tabs to show
     buttonGroup = new QGroupBox(i18n("&Tabs"), generalFrame);
     vbox = new QVBoxLayout;
@@ -271,6 +523,31 @@ void KateConfigDialog::addBehaviorPage()
 
     buttonGroup->setLayout(vbox);
 
+    /** DIFF **/
+    buttonGroup = new QGroupBox(i18n("Diff"), generalFrame);
+    vbox = new QVBoxLayout(buttonGroup);
+    hlayout = new QHBoxLayout;
+    vbox->addLayout(hlayout);
+    layout->addWidget(buttonGroup);
+    m_diffStyle = new QComboBox;
+    m_diffStyle->addItem(i18n("Side By Side"));
+    m_diffStyle->addItem(i18n("Unified"));
+    m_diffStyle->addItem(i18n("Raw"));
+    hlayout->addWidget(new QLabel(i18n("Diff Style:")));
+    hlayout->addWidget(m_diffStyle);
+    m_diffStyle->setCurrentIndex(cgGeneral.readEntry("Diff Show Style", 0));
+    connect(m_diffStyle, static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged), this, &KateConfigDialog::slotChanged);
+
+    buttonGroup = new QGroupBox(i18n("Navigation Bar"), generalFrame);
+    vbox = new QVBoxLayout(buttonGroup);
+    hlayout = new QHBoxLayout;
+    vbox->addLayout(hlayout);
+    layout->addWidget(buttonGroup);
+    m_urlBarShowSymbols = new QCheckBox(i18n("Show current symbol in navigation bar"));
+    hlayout->addWidget(m_urlBarShowSymbols);
+    m_urlBarShowSymbols->setChecked(cgGeneral.readEntry("Show Symbol In Navigation Bar", true));
+    connect(m_urlBarShowSymbols, &QCheckBox::toggled, this, &KateConfigDialog::slotChanged);
+
     layout->addStretch(1); // :-] works correct without autoadd
 }
 
@@ -281,6 +558,7 @@ void KateConfigDialog::addSessionPage()
 
     QWidget *sessionsPage = new QWidget();
     auto item = addScrollablePage(sessionsPage, i18n("Session"));
+    m_allPages.insert(item);
     item->setHeader(i18n("Session Management"));
     item->setIcon(QIcon::fromTheme(QStringLiteral("view-history")));
 
@@ -320,6 +598,15 @@ void KateConfigDialog::addSessionPage()
     connect(sessionConfigUi.loadLastUserSessionRadioButton, &QRadioButton::toggled, this, &KateConfigDialog::slotChanged);
     connect(sessionConfigUi.manuallyChooseSessionRadioButton, &QRadioButton::toggled, this, &KateConfigDialog::slotChanged);
 
+    // New main windows open always a new document if none there
+    sessionConfigUi.showWelcomeViewForNewWindow->setChecked(cgGeneral.readEntry("Show welcome view for new window", true));
+    connect(sessionConfigUi.showWelcomeViewForNewWindow, &QCheckBox::toggled, this, &KateConfigDialog::slotChanged);
+
+    // When a window is closed, close all documents only visible in that window, too
+    sessionConfigUi.winClosesDocuments->setChecked(cgGeneral.readEntry("Close documents with window", true));
+    sessionConfigUi.winClosesDocuments->setToolTip(i18n("When a window is closed the documents opened only in this window are closed as well."));
+    connect(sessionConfigUi.winClosesDocuments, &QCheckBox::toggled, this, &KateConfigDialog::slotChanged);
+
     // Closing last file closes Kate
     sessionConfigUi.modCloseAfterLast->setChecked(m_mainWindow->modCloseAfterLast());
     connect(sessionConfigUi.modCloseAfterLast, &QCheckBox::toggled, this, &KateConfigDialog::slotChanged);
@@ -353,6 +640,7 @@ void KateConfigDialog::addPluginsPage()
     connect(configPluginPage, &KateConfigPluginPage::changed, this, &KateConfigDialog::slotChanged);
 
     auto item = addScrollablePage(page, i18n("Plugins"));
+    m_allPages.insert(item);
     item->setHeader(i18n("Plugin Manager"));
     item->setIcon(QIcon::fromTheme(QStringLiteral("preferences-plugin")));
 }
@@ -372,6 +660,7 @@ void KateConfigDialog::addFeedbackPage()
     vlayout->addWidget(m_userFeedbackWidget);
 
     auto item = addScrollablePage(page, i18n("User Feedback"));
+    m_allPages.insert(item);
     item->setHeader(i18n("User Feedback"));
     item->setIcon(QIcon::fromTheme(QStringLiteral("preferences-desktop-locale")));
 #endif
@@ -396,6 +685,7 @@ void KateConfigDialog::addEditorPages()
         KPageWidgetItem *item = addScrollablePage(page, page->name());
         item->setHeader(page->fullName());
         item->setIcon(page->icon());
+        m_allPages.insert(item);
     }
 }
 
@@ -414,6 +704,7 @@ void KateConfigDialog::addPluginPage(KTextEditor::Plugin *plugin)
         info.pageWidgetItem = item;
         connect(info.pluginPage, &KTextEditor::ConfigPage::changed, this, &KateConfigDialog::slotChanged);
         m_pluginPages.insert(item, info);
+        m_allPages.insert(item);
     }
 }
 
@@ -432,6 +723,7 @@ void KateConfigDialog::removePluginPage(KTextEditor::Plugin *plugin)
     while (!remove.isEmpty()) {
         KPageWidgetItem *wItem = remove.takeLast();
         PluginPageListItem item = m_pluginPages.take(wItem);
+        m_allPages.remove(wItem);
         delete item.pluginPage;
         removePage(wItem);
     }
@@ -446,6 +738,9 @@ void KateConfigDialog::slotApply()
         KConfigGroup cg(config, "General");
 
         // only there for kate
+        if (m_syncSectionSizeWithSidebarTabs) {
+            cg.writeEntry("Sync section size with tab positions", m_syncSectionSizeWithSidebarTabs->isChecked());
+        }
         if (m_showTextForLeftRightSidebars) {
             cg.writeEntry("Show text for left and right sidebar", m_showTextForLeftRightSidebars->isChecked());
         }
@@ -471,6 +766,10 @@ void KateConfigDialog::slotApply()
         cg.writeEntry("Days Meta Infos", sessionConfigUi.daysMetaInfos->value());
         KateApp::self()->documentManager()->setDaysMetaInfos(sessionConfigUi.daysMetaInfos->value());
 
+        cg.writeEntry("Show welcome view for new window", sessionConfigUi.showWelcomeViewForNewWindow->isChecked());
+
+        cg.writeEntry("Close documents with window", sessionConfigUi.winClosesDocuments->isChecked());
+
         cg.writeEntry("Close After Last", sessionConfigUi.modCloseAfterLast->isChecked());
         m_mainWindow->setModCloseAfterLast(sessionConfigUi.modCloseAfterLast->isChecked());
 
@@ -504,6 +803,10 @@ void KateConfigDialog::slotApply()
         cg.writeEntry("Allow Tab Scrolling", m_tabsScrollable->isChecked());
         cg.writeEntry("Elide Tab Text", m_tabsElided->isChecked());
 
+        cg.writeEntry("Diff Show Style", m_diffStyle->currentIndex());
+
+        cg.writeEntry("Show Symbol In Navigation Bar", m_urlBarShowSymbols->isChecked());
+
         // patch document modified warn state
         const QList<KTextEditor::Document *> &docs = KateApp::self()->documentManager()->documentList();
         for (KTextEditor::Document *doc : docs) {
@@ -546,6 +849,8 @@ void KateConfigDialog::slotApply()
 
     m_dataChanged = false;
     buttonBox()->button(QDialogButtonBox::Apply)->setEnabled(false);
+
+    Q_EMIT saved();
 }
 
 void KateConfigDialog::slotChanged()
@@ -575,6 +880,18 @@ int KateConfigDialog::recentFilesMaxCount()
     return maxItems;
 }
 
+bool KateConfigDialog::event(QEvent *event)
+{
+    if (event->type() == QEvent::ShortcutOverride || event->type() == QEvent::KeyPress) {
+        auto keyEvent = static_cast<QKeyEvent *>(event);
+        if (keyEvent->modifiers().testFlag(Qt::NoModifier) && keyEvent->key() == Qt::Key_Escape) {
+            event->accept();
+            return true;
+        }
+    }
+    return KPageDialog::event(event);
+}
+
 void KateConfigDialog::closeEvent(QCloseEvent *event)
 {
     if (!m_dataChanged) {
@@ -582,17 +899,29 @@ void KateConfigDialog::closeEvent(QCloseEvent *event)
         return;
     }
 
+#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
+    const auto response = KMessageBox::warningTwoActionsCancel(this,
+#else
     const auto response = KMessageBox::warningYesNoCancel(this,
-                                                          i18n("You have unsaved changes. Do you want to apply the changes or discard them?"),
-                                                          i18n("Warning"),
-                                                          KStandardGuiItem::save(),
-                                                          KStandardGuiItem::discard(),
-                                                          KStandardGuiItem::cancel());
+#endif
+                                                               i18n("You have unsaved changes. Do you want to apply the changes or discard them?"),
+                                                               i18n("Warning"),
+                                                               KStandardGuiItem::save(),
+                                                               KStandardGuiItem::discard(),
+                                                               KStandardGuiItem::cancel());
     switch (response) {
+#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
+    case KMessageBox::PrimaryAction:
+#else
     case KMessageBox::Yes:
+#endif
         slotApply();
         Q_FALLTHROUGH();
+#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
+    case KMessageBox::SecondaryAction:
+#else
     case KMessageBox::No:
+#endif
         event->accept();
         break;
     case KMessageBox::Cancel:
