@@ -9,6 +9,7 @@
 #ifndef __KATE_MAINWINDOW_H__
 #define __KATE_MAINWINDOW_H__
 
+#include "diffparams.h"
 #include "katemdi.h"
 #include "kateviewmanager.h"
 
@@ -23,11 +24,8 @@
 #include <QDropEvent>
 #include <QEvent>
 #include <QHash>
-#include <QModelIndex>
 #include <QStackedLayout>
-#include <QStackedWidget>
 #include <QUrl>
-#include <QVBoxLayout>
 
 class QMenu;
 
@@ -119,7 +117,7 @@ public:
     void loadOpenRecent(const KConfig *config);
 
 public:
-    bool queryClose_internal(KTextEditor::Document *doc = nullptr);
+    bool queryClose_internal(KTextEditor::Document *doc = nullptr, KateMainWindow *win = nullptr);
 
     /**
      * save the settings, size and state of this window in
@@ -137,6 +135,15 @@ public:
      */
     void saveOptions();
 
+    /**
+     * Called if focus should go to the important part of the central widget.
+     */
+    void triggerFocusForCentralWidget() override
+    {
+        // just dispatch to view manager
+        m_viewManager->triggerActiveViewFocus();
+    }
+
 private:
     /**
      * Setup actions which pointers are needed already in setupMainWindow
@@ -187,7 +194,6 @@ private Q_SLOTS:
 
     void slotEditToolbars();
     void slotNewToolbarConfig();
-    void slotUpdateOpenWith();
     void slotUpdateActionsNeedingUrl();
     void slotOpenDocument(const QUrl &);
 
@@ -220,6 +226,9 @@ Q_SIGNALS:
     void tabBarToggled();
     void unhandledShortcutOverride(QEvent *e);
 
+    void widgetAdded(QWidget *);
+    void widgetRemoved(QWidget *);
+
 public:
     void openUrl(const QString &name = QString());
 
@@ -299,6 +308,11 @@ public:
      */
     void addRecentOpenedFile(const QUrl &url);
 
+    KRecentFilesAction *recentFilesAction()
+    {
+        return m_fileOpenRecent;
+    }
+
     //
     // KTextEditor::MainWindow interface, get called by invokeMethod from our wrapper object!
     //
@@ -511,7 +525,48 @@ public Q_SLOTS:
      */
     QObject *pluginView(const QString &name);
 
-    void addWidgetAsTab(QWidget *widget);
+    /**
+     * Add a widget to the main window.
+     * This is useful to show non-KTextEditor::View widgets in the main window.
+     * The host application should try to manage this like some KTextEditor::View (e.g. as a tab) and provide
+     * the means to close it.
+     * \param widget widget to add
+     * \return success, if false, the plugin needs to take care to show the widget itself, otherwise
+     *         the main window will take ownership of the widget
+     */
+    bool addWidget(QWidget *widget);
+
+    /**
+     * \brief returns the list of non-KTextEditor::View widgets in this main window.
+     * \see addWidget
+     */
+    QWidgetList widgets() const;
+
+    /**
+     * \brief remove this \p widget from this mainwindow. The widget will be deleted afterwards
+     * \param widget the widget to be removed
+     * \return true on success
+     */
+    bool removeWidget(QWidget *widget);
+
+    /**
+     * \brief returns the currently active widget. It can be a
+     * non-KTextEditor::View widget or a KTextEditor::View
+     */
+    QWidget *activeWidget();
+
+    /**
+     * \brief activate @p widget. If the widget is not present in the window, it will be added to the window
+     * \param widget the widget to activate
+     */
+    void activateWidget(QWidget *widget);
+
+    void showDiff(const QByteArray &wordDiff, const DiffParams &params);
+
+    /**
+     * @brief show a message in the output view. See KTextEditor::MainWindow::showMessage for details
+     */
+    void showMessage(const QVariantMap &map);
 
 private Q_SLOTS:
     void slotUpdateBottomViewBar();
@@ -533,11 +588,6 @@ private:
      */
     bool m_modCloseAfterLast = false;
 
-    /**
-     * stacked widget containing the central area, aka view manager, quickopen, ...
-     */
-    QStackedWidget *m_mainStackedWidget = nullptr;
-
     /**
      * keeps track of views
      */
