@@ -79,6 +79,7 @@ private:
     KActionMenu *m_toolMenu;
     QAction *m_hideToolViews;
     KActionMenu *m_sidebarButtonsMenu;
+    KActionMenu *m_focusToolviewMenu;
 };
 
 class ToolView : public QFrame
@@ -86,6 +87,7 @@ class ToolView : public QFrame
     Q_OBJECT
 
     friend class Sidebar;
+    friend class MultiTabBar;
     friend class MainWindow;
     friend class GUIClient;
     friend class ToggleToolViewAction;
@@ -99,8 +101,9 @@ protected:
      * @param mainwin main window for this toolview
      * @param sidebar sidebar of this toolview
      * @param parent parent widget, e.g. the splitter of one of the sidebars
+     * @param identifier unique id
      */
-    ToolView(class MainWindow *mainwin, class Sidebar *sidebar, QWidget *parent);
+    ToolView(class MainWindow *mainwin, class Sidebar *sidebar, QWidget *parent, const QString &identifier);
 
 public:
     /**
@@ -161,7 +164,7 @@ private:
     /**
      * unique id
      */
-    QString id;
+    const QString id;
 
     /**
      * is visible in sidebar
@@ -182,52 +185,126 @@ private:
     QString text;
 };
 
-class Sidebar : public KMultiTabBar
+class MultiTabBar : public QWidget
 {
     Q_OBJECT
 
 public:
-    Sidebar(KMultiTabBar::KMultiTabBarPosition pos, class MainWindow *mainwin, QWidget *parent);
+    MultiTabBar(KMultiTabBar::KMultiTabBarPosition pos, Sidebar *sb, int idx);
+    ~MultiTabBar();
 
-    void setSplitter(QSplitter *sp);
+    KMultiTabBarTab *addTab(int id, ToolView *tv);
+    int addBlankTab();
+    void removeBlankTab(int id);
+    void removeTab(int id);
+
+    void showToolView(int id);
+    void hideToolView(int id);
+
+    void setTabActive(int id, bool state);
+
+    bool isToolActive() const;
+    void collapseToolView() const;
+    bool expandToolView() const;
+
+    KMultiTabBar *tabBar() const
+    {
+        return m_multiTabBar;
+    }
+
+    QList<int> tabList() const
+    {
+        return m_tabList;
+    }
+
+    int tabCount() const
+    {
+        return m_tabList.size();
+    }
+
+    int sectionSize() const
+    {
+        return m_sectionSize;
+    }
+
+    void setSectionSize(int size)
+    {
+        m_sectionSize = size;
+    }
+
+Q_SIGNALS:
+    void lastTabRemoved(MultiTabBar *);
+
+private Q_SLOTS:
+    void tabClicked(int);
+
+private:
+    Sidebar *m_sb;
+    QStackedWidget *m_stack;
+    KMultiTabBar *m_multiTabBar;
+    QList<int> m_tabList;
+    int m_activeTab = 0;
+    int m_sectionSize = 0;
+};
+
+class Sidebar : public QSplitter
+{
+    Q_OBJECT
+
+    friend class MultiTabBar;
+
+public:
+    Sidebar(KMultiTabBar::KMultiTabBarPosition pos, QSplitter *sp, class MainWindow *mainwin, QWidget *parent);
 
     QSize sizeHint() const override;
 
-    /**
-     * Monitor resizes using the mouse and update the last size accordingly.
-     * Only call this when the sidebar has siblings in the splitter (i.e. m_splitter->count() >= 2) to guarantee that resize handles exist
-     */
-    void updateLastSizeOnResize();
+    ToolView *addToolView(const QIcon &icon, const QString &text, const QString &identifier, ToolView *widget);
+    bool removeToolView(ToolView *widget);
 
-public:
-    ToolView *addWidget(const QIcon &icon, const QString &text, ToolView *widget);
-    bool removeWidget(ToolView *widget);
-
-    bool showWidget(ToolView *widget);
-    bool hideWidget(ToolView *widget);
+    bool showToolView(ToolView *widget);
+    bool hideToolView(ToolView *widget);
 
     void showToolviewTab(ToolView *widget, bool show);
 
     bool isCollapsed();
-    void handleCollapse(int pos, int index);
-    void expandSidebar(ToolView *widget);
 
-    void setLastSize(int s)
+    QWidget *tabButtonForToolview(ToolView *widget) const
     {
-        m_lastSize = s;
-    }
-    int lastSize() const
-    {
-        return m_lastSize;
-    }
-    void updateLastSize();
-
-    bool splitterVisible() const
-    {
-        return m_ownSplit->isVisible();
+        auto it = m_widgetToId.find(widget);
+        if (it == m_widgetToId.end()) {
+            return nullptr;
+        }
+        if (auto tabbar = kmTabBar(widget)) {
+            return tabbar->tab(it->second);
+        }
+        return nullptr;
     }
 
-    void restoreSession();
+    /**
+     * Will the sidebar expand when some tool has to be visible in any section,
+     * or calling collapseSidebar() if non such tool is found
+     */
+    void updateSidebar();
+    void collapseSidebar();
+
+    KMultiTabBar::KMultiTabBarPosition position() const
+    {
+        return m_tabBarPosition;
+    }
+
+    bool isVertical() const
+    {
+        return m_tabBarPosition == KMultiTabBar::Right || m_tabBarPosition == KMultiTabBar::Left;
+    }
+
+    void setStyle(KMultiTabBar::KMultiTabBarStyle style);
+
+    KMultiTabBar::KMultiTabBarStyle tabStyle() const
+    {
+        return m_tabBarStyle;
+    }
+
+    void startRestoreSession(KConfigGroup &config);
 
     /**
      * restore the current session config from given object, use current group
@@ -244,55 +321,92 @@ public:
 public Q_SLOTS:
     // reimplemented, to block a show() call if all sidebars are forced hidden
     void setVisible(bool visible) override;
-private Q_SLOTS:
-    void tabClicked(int);
-    void readConfig();
-
-private:
-    /**
-     * Append a tab with our styling & needed connections/event filter.
-     */
-    void appendStyledTab(const QIcon &icon, int id, const QString &text);
-
-    /**
-     * Update style of button to our style.
-     */
-    void updateButtonStyle(KMultiTabBarTab *button);
 
 protected:
     bool eventFilter(QObject *obj, QEvent *ev) override;
 
 private Q_SLOTS:
     void buttonPopupActivate(QAction *);
+    void readConfig();
+    void handleCollapse(int pos, int index);
+    void ownSplitMoved(int pos, int index);
+    void barSplitMoved(int pos, int index);
+    bool tabBarIsEmpty(MultiTabBar *bar);
 
 private:
+    void updateLastSize();
+    int nextId();
+    bool adjustSplitterSections();
+
+    /**
+     * Append a tab with our styling & needed connections/event filter.
+     */
+    void appendStyledTab(int id, MultiTabBar *bar, ToolView *widget);
+
+    /**
+     * Update style of button to our style.
+     */
+    void updateButtonStyle(KMultiTabBarTab *button);
+
+    /**
+     * Monitor resizes using the mouse and update the last size accordingly.
+     */
+    void updateLastSizeOnResize();
+
+    MultiTabBar *insertTabBar(int idx = -1);
+
+    MultiTabBar *tabBar(int idx) const
+    {
+        return static_cast<MultiTabBar *>(widget(idx));
+    }
+
+    MultiTabBar *tabBar(ToolView *tv) const
+    {
+        return m_widgetToTabBar.at(tv);
+    }
+
+    KMultiTabBar *kmTabBar(ToolView *widget) const
+    {
+        return m_widgetToTabBar.at(widget)->tabBar();
+    }
+
+    int tabBarCount() const
+    {
+        return count();
+    }
+
+private:
+    enum ActionIds {
+        HideButtonAction = 11,
+        ConfigureAction = 20,
+        ToOwnSectAction = 30,
+        UpLeftAction = 31,
+        DownRightAction = 32,
+    };
+
     MainWindow *m_mainWin;
 
-    KMultiTabBar::KMultiTabBarPosition m_pos{};
+    KMultiTabBar::KMultiTabBarPosition m_tabBarPosition{};
+    KMultiTabBar::KMultiTabBarStyle m_tabBarStyle{};
     QSplitter *m_splitter;
-    KMultiTabBar *m_tabBar = nullptr;
     QSplitter *m_ownSplit;
+    const int m_ownSplitIndex;
 
     std::map<int, ToolView *> m_idToWidget;
     std::map<ToolView *, int> m_widgetToId;
-    std::map<ToolView *, QSize> m_widgetToSize;
+    std::map<ToolView *, MultiTabBar *> m_widgetToTabBar;
 
-    /**
-     * list of all toolviews around in this sidebar
-     */
-    std::vector<ToolView *> m_toolviews;
+    // Session restore only
+    std::map<QString, int> m_tvIdToTabId;
+    std::map<QString, int> m_tvIdToTabBar;
+    bool m_sessionRestoreRunning = false;
 
     int m_lastSize;
-
-    QSize m_preHideSize;
-
     int m_popupButton = 0;
-
-    QLabel *m_resizePlaceholder;
+    QPointer<QLabel> m_resizePlaceholder;
     bool m_isPreviouslyCollapsed = false;
-
+    bool m_syncWithTabs = false;
     bool m_showTextForLeftRight = false;
-
     int m_leftRightSidebarIconSize = 32;
 
 Q_SIGNALS:
@@ -342,19 +456,6 @@ public:
      */
     ToolView *toolView(const QString &identifier) const;
 
-    /**
-     * set the toolview's tabbar style.
-     * @param style the tabbar style.
-     */
-    void setToolViewStyle(KMultiTabBar::KMultiTabBarStyle style);
-
-    /**
-     * get the toolview's tabbar style. Call this before @p startRestore(),
-     * otherwise you overwrite the usersettings.
-     * @return toolview's tabbar style
-     */
-    KMultiTabBar::KMultiTabBarStyle toolViewStyle() const;
-
     /**
      * get the sidebars' visibility.
      * @return false, if the sidebars' visibility is forced hidden, otherwise true
@@ -394,14 +495,36 @@ protected:
      */
     void toolViewDeleted(ToolView *widget);
 
+    /**
+     * set the toolview's tabbar style.
+     * @param style the tabbar style.
+     */
+    void setToolViewStyle(KMultiTabBar::KMultiTabBarStyle style);
+
+    /**
+     * get the toolview's tabbar style. Call this before @p startRestore(),
+     * otherwise you overwrite the usersettings.
+     * @return toolview's tabbar style
+     */
+    KMultiTabBar::KMultiTabBarStyle toolViewStyle() const;
+
+public:
     /**
      * central widget ;)
      * use this as parent for your content
-     * this widget will get focus if a toolview is hidden
      * @return central widget
      */
     QWidget *centralWidget() const;
 
+    /**
+     * Called if focus should go to the important part of the central widget.
+     * Default doesn't do a thing, if we are called in destructor by accident.
+     */
+    virtual void triggerFocusForCentralWidget()
+    {
+    }
+
+protected:
     /**
      * Status bar area stacked widget.
      * We plug in our status bars from the KTextEditor::Views here
@@ -437,6 +560,8 @@ public:
      */
     bool hideToolView(ToolView *widget);
 
+    QWidget *toolviewToggleButton(ToolView *tv);
+
     /**
      * session saving and restore stuff
      */
@@ -464,14 +589,11 @@ public:
      */
 private:
     /**
-     * map identifiers to widgets
+     * all existing tool views
+     * mapped by their constant identifier, to have some stable order
+     * tool views de-register them self on destruction
      */
-    std::map<QString, QPointer<ToolView>> m_idToWidget;
-
-    /**
-     * list of all toolviews around
-     */
-    std::vector<QPointer<ToolView>> m_toolviews;
+    std::map<QString, ToolView *> m_toolviews;
 
     /**
      * widget, which is the central part of the
