@@ -9,12 +9,13 @@
 #ifndef __KATE_VIEWMANAGER_H__
 #define __KATE_VIEWMANAGER_H__
 
+#include "doc_or_widget.h"
 #include "katedocmanager.h"
 #include "kateprivate_export.h"
 #include "katesplitter.h"
+#include "kateviewspace.h"
 
 #include <QList>
-#include <QMap>
 #include <QPointer>
 
 #include <unordered_map>
@@ -41,10 +42,17 @@ class KATE_PRIVATE_EXPORT KateViewManager : public KateSplitter
 {
     Q_OBJECT
 
+    friend class KateViewManagementTests;
+
 public:
     KateViewManager(QWidget *parentW, KateMainWindow *parent);
     ~KateViewManager() override;
 
+    /**
+     * triggered delayed focus set to current active view
+     */
+    void triggerActiveViewFocus();
+
 private:
     /**
      * create all actions needed for the view manager
@@ -70,15 +78,18 @@ public:
 
     KTextEditor::View *openUrlWithView(const QUrl &url, const QString &encoding);
 
+    KTextEditor::View *openViewForDoc(KTextEditor::Document *doc);
+
 public Q_SLOTS:
     void openUrl(const QUrl &url);
+    void openUrlOrProject(const QUrl &url);
     void addPositionToHistory(const QUrl &url, KTextEditor::Cursor pos);
 
 public:
     void closeView(KTextEditor::View *view);
     KateMainWindow *mainWindow();
 
-    void moveViewToViewSpace(KateViewSpace *dest, KateViewSpace *src, KTextEditor::Document *doc);
+    void moveViewToViewSpace(KateViewSpace *dest, KateViewSpace *src, DocOrWidget doc);
 
 private Q_SLOTS:
     void activateView(KTextEditor::View *view);
@@ -91,10 +102,17 @@ public Q_SLOTS:
     void slotDocumentClose(KTextEditor::Document *document);
 
     void setActiveSpace(KateViewSpace *vs);
-    void setActiveView(KTextEditor::View *view);
 
     void activateNextView();
     void activatePrevView();
+    void activateLeftView();
+    void activateRightView();
+    void activateUpwardView();
+    void activateDownwardView();
+
+private:
+    void activateIntuitiveNeighborView(Qt::Orientation o, int dir);
+    KateViewSpace *findIntuitiveNeighborView(KateSplitter *splitter, QWidget *widget, Qt::Orientation o, int dir);
 
 Q_SIGNALS:
     void viewChanged(KTextEditor::View *);
@@ -136,10 +154,14 @@ public:
     KTextEditor::View *activeView();
     KateViewSpace *activeViewSpace();
 
+    /// Returns all non-KTE::View widgets in this viewManager
+    QWidgetList widgets() const;
+    bool removeWidget(QWidget *w);
+    bool activateWidget(QWidget *w);
+
 private Q_SLOTS:
     void slotViewChanged();
 
-    void documentCreated(KTextEditor::Document *doc);
     void documentWillBeDeleted(KTextEditor::Document *doc);
 
     void documentSavedOrUploaded(KTextEditor::Document *document, bool saveAs);
@@ -191,7 +213,7 @@ public Q_SLOTS:
      * activate view for given document
      * @param doc document to activate view for
      */
-    KTextEditor::View *activateView(KTextEditor::Document *doc);
+    KTextEditor::View *activateView(DocOrWidget docOrWidget);
 
     /** Splits the active viewspace horizontally */
     void slotSplitViewSpaceHoriz()
@@ -256,7 +278,7 @@ public Q_SLOTS:
     /** hide every view but the active one */
     void slotHideOtherViews(bool hideOthers);
 
-    void reactivateActiveView();
+    void replugActiveView();
 
     /**
      * Toogle the orientation of current split view
@@ -291,9 +313,20 @@ public Q_SLOTS:
 
     void onViewSpaceEmptied(KateViewSpace *vs);
 
+    // Returns the number of viewspaces this document is registered in
+    int viewspaceCountForDoc(KTextEditor::Document *doc) const;
+    // returns true if @p doc exists only in one viewspace
+    bool docOnlyInOneViewspace(KTextEditor::Document *doc) const;
+    // returns true if any viewspace in this viewmanager has its tabbar visible
+    bool tabsVisible() const;
+
     void setShowUrlNavBar(bool show);
     bool showUrlNavBar() const;
 
+    void hideWelcomeView(KateViewSpace *vs);
+    void showWelcomeViewOrNewDocumentIfNeeded();
+    void showWelcomeView();
+
 private:
     KateMainWindow *m_mainWindow;
 
@@ -307,13 +340,15 @@ private:
     QAction *m_hideOtherViews = nullptr;
     QAction *goNext = nullptr;
     QAction *goPrev = nullptr;
+    QAction *goLeft = nullptr;
+    QAction *goRight = nullptr;
+    QAction *goUp = nullptr;
+    QAction *goDown = nullptr;
 
     std::vector<KateViewSpace *> m_viewSpaceList;
 
     bool m_blockViewCreationAndActivation;
 
-    bool m_activeViewRunning;
-
     bool m_showUrlNavBar = false;
 
     int m_splitterIndex = 0; // used during saving splitter config.
@@ -324,11 +359,6 @@ private:
     class ViewData
     {
     public:
-        /**
-         * view active?
-         */
-        bool active = false;
-
         /**
          * lru age of the view
          * important: smallest age ==> latest used view
